# coding=utf-8
import os

from earthdiagnostics import cdftools
from earthdiagnostics.diagnostic import Diagnostic
from earthdiagnostics.utils import Utils, TempFile
from earthdiagnostics.modelingrealm import ModelingRealms


class Mxl(Diagnostic):
    """
    Compute the mixed layer depth

    :param data_manager: data management object
    :type data_manager: DataManager
    :param startdate: startdate
    :type startdate: str
    :param member: member number
    :type member: int
    :param chunk: chunk's number
    :type chunk: int
    """

    alias = 'mxl'
    "Diagnostic alias for the configuration file"

    def __init__(self, data_manager, startdate, member, chunk):
        Diagnostic.__init__(self, data_manager)
        self.startdate = startdate
        self.member = member
        self.chunk = chunk

    def __eq__(self, other):
        return self.startdate == other.startdate and self.member == other.member and self.chunk == other.chunk

    def __str__(self):
        return 'Mixed layer Startdate: {0} Member: {1} Chunk: {2}'.format(self.startdate, self.member, self.chunk)

    @classmethod
    def generate_jobs(cls, diags, options):
        """
        Creates a job for each chunk to compute the diagnostic

        :param diags: Diagnostics manager class
        :type diags: Diags
        :param options: None
        :type options: list[str]
        :return:
        """
        if len(options) > 1:
            raise Exception('The mxl diagnostic has no options')
        job_list = list()
        for startdate, member, chunk in diags.config.experiment.get_chunk_list():
            job_list.append(Mxl(diags.data_manager, startdate, member, chunk))
        return job_list

    def compute(self):
        """
        Runs the diagnostic
        """
        temp = TempFile.get()
        thetao = self.data_manager.get_file(ModelingRealms.ocean, 'thetao', self.startdate, self.member, self.chunk)
        so = self.data_manager.get_file(ModelingRealms.ocean, 'so', self.startdate, self.member, self.chunk)
        cdftools.run('cdfmxl', input=[thetao, so], output=temp, options='-nc4')
        os.remove(thetao)
        os.remove(so)
        temp2 = TempFile.get()
        source = Utils.openCdf(temp)
        destiny = Utils.openCdf(temp2, 'w')
        Utils.copy_variable(source, destiny, 'somxl010', must_exist=True, add_dimensions=True)
        Utils.copy_variable(source, destiny, 'lat')
        Utils.copy_variable(source, destiny, 'lon')
        source.close()
        destiny.close()
        self.send_file(temp2, ModelingRealms.ocean, 'mlotst', self.startdate, self.member, self.chunk,
                       rename_var='somxl010')
        os.remove(temp)
